'use strict';

var fs = require('fs');

var ischanged = require('ischanged');
var FILE = '/etc/passwd';

var DARWIN = process.platform === 'darwin';

var Names;

module.exports = function (callback) {
    if (DARWIN) {
        if (Names)
            { return callback(null, Names); }
       
        require('./darwin')(function (error, data) {
            Names = data;
            callback(error, data);
        });
    } else {
        ischanged(FILE, function (error, is) {
            if (error)
                { return callback(error); }
            
            if (is || !Names)
                { return read(callback); }
            
            callback(null, Names);
        });
    }
};

module.exports.parse = get;

function read(callback) {
    fs.readFile(FILE, 'utf8', function (error, passwd) {
        if (!error)
            { Names = get(passwd); }
        
        callback(error, Names);
    });
}


/** Функция парсит uid и имена пользователей
 * из переданного в строке вычитаного файла /etc/passwd
 * и возвращает массив обьектов имён и uid пользователей
 * @param passwd - строка, в которой находиться файл /etc/passwd
 */
function get(passwd) {
    var users   = {};
    
    passwd
        .split('\n')
        .forEach(function (line) {
            passwd = passwd.replace(line, '');
            
            /* получаем первое слово строки */
            var name = line.substr(line, line.indexOf(':'));
            
            line = line.replace(name + ':x:', '');
            
            /* получаем uid */
            var uid = line.substr(line, line.indexOf(':'));
            
            if (!uid)
                { return; }
            
            users[uid] = name;
        });
    
    return users;
}

